# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

import unittest
import boots_unit_test
from boots.lib.ui import generic, plain
from boots.lib import console
from boots.app import client_config
from boots.api import api
import StringIO

class TestGeneric(boots_unit_test.BootsBaseTester):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_example(self):
        # TODO: test me!
        pass

class TestPlain(boots_unit_test.BootsBaseTester):
    def setUp(self):
        self.config = client_config.ClientConfig()
        self.console = console.Console(self.config)
        self.plain_ui = plain.PlainUI(self.console)

    def tearDown(self):
        pass

    def test_show_NULL(self):
        """_show_NULL() should return whatever it's given unless
        it's given nothing/None, in which case it should return
        the string 'NULL'."""
        self.assertEqual(self.plain_ui._show_NULL(None), "NULL")
        self.assertEqual(self.plain_ui._show_NULL(0), 0)
        self.assertEqual(self.plain_ui._show_NULL("0"), "0")
        self.assertEqual(self.plain_ui._show_NULL(1), 1)
        self.assertEqual(self.plain_ui._show_NULL("1"), "1")
        self.assertEqual(self.plain_ui._show_NULL("hello, world!"), "hello, world!")

    def test_print_notificatilon(self):
        """By default print_notification() writes a message to STDOUT
        like the one below.  For testing, we pass a third arg, a StringIO
        to capture the output."""
        output = StringIO.StringIO()
        self.plain_ui.print_notification("title", "here's my message", output)
        notif = output.getvalue()
        output.close()
        self.assertEqual(notif, "title :: here's my message\n")

    def test_gen_output(self):
        """_gen_output() outputs rows in either default table/horizontal
        format or optional vertical (\G) format.  It returns a generator
        that yields output lines."""

        # Add a row so _gen_output() has something to output.
        row_desc = api.RowDescription(
            (('Databases', 15, 3, 256, 256, 0, 0),))
        rows = api.Rows( (('db1',),), row_desc)
        self.plain_ui.present(rows)

        # Result info for the row.
        info = api.ResultInfo({
            "description": row_desc,
            "row_count": 1,
            "begin_time": 1.0,
            "end_time": 3.0,
            "server_elapsed": 1.0,
        })

        # Test default table/horizontal output.
        res = self.plain_ui._gen_output(info.value)
        lines = []
        for line in res:
            lines.append(line)
        self.assertEqual(lines, [
            "+-----------+",
            "| Databases |",
            "+-----------+",
            "| db1       |",
            "+-----------+",
            "1 row in set (1.00s server | +1.00s working)",
        ])

        # Test optional veritcal output.
        self.plain_ui.vertical_output = True
        res = self.plain_ui._gen_output(info.value)
        lines = []
        for line in res:
            lines.append(line)
        self.assertEqual(lines, [
            "=================================== Row 1 ====================================",
            "Databases: db1",
            "",
            "1 row in set (1.00s server | +1.00s working)",
        ])
